
#include "resourcemanager.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

int cResourceManager::Init()
{
    AudioInit();

    ImgInit();

	printf("cResourceManager Init\n");

	return 0;
}

void cResourceManager::Cleanup()
{
	printf("ResourceEngine Cleanup\n");

	AudioExit();

	ImgExit();
}

void cResourceManager::AudioInit()
{
    Mix_OpenAudio( 22050, MIX_DEFAULT_FORMAT, 2, 4096 );

    //uzmi radni direktorijum
    char putanja[256];
    getcwd(putanja, 255);
    strcat(putanja, "/");
    strcat(putanja, "sndres001.bin");

    r_Prijava = UzmiZvuk(putanja, "prijava.wav");
    r_Odjava = UzmiZvuk(putanja, "odjava.wav");
    r_Klik = UzmiZvuk(putanja, "klik.wav");
    r_Aplauz = UzmiZvuk(putanja, "aplauz.wav");
    r_Ubac = UzmiZvuk(putanja, "ubac.wav");
    r_PokusajPonovo = UzmiZvuk(putanja, "ponovo.wav");
    r_Muzika = UzmiMuziku(putanja, "aladin.mid");
}

void cResourceManager::AudioExit()
{
    Mix_FreeChunk( r_Prijava );
    Mix_FreeChunk( r_Klik );
    Mix_PlayChannel( -1, r_Odjava, 0 );
    SDL_Delay(1000);
    Mix_FreeChunk( r_Odjava );
    Mix_FreeMusic( r_Muzika );
    Mix_CloseAudio();
}

void cResourceManager::ImgInit()
{
    //uzmi radni direktorijum
    char putanja[256];
    getcwd(putanja, 255);
    strcat(putanja, "/");
    strcat(putanja,"imgres001.bin");

    r_IntroPozadina = UzmiBitmap(putanja, "sivo.gif");
    r_DugmeStart    = UzmiBitmap(putanja, "start_sprajtmapa.gif");
    r_IzborPozadina = UzmiBitmap(putanja, "pejzaz-s.gif");
    r_DugmeIzbor    = UzmiBitmap(putanja, "serijacija_ikonica_sprajtmapa.gif");
    r_StapiciPozadina = UzmiBitmap(putanja, "sss_bez_balvana.gif");
    r_StapiciBitmap = UzmiBitmap(putanja, "balvani_sprajtmapa.gif");
    r_DugmeIzlaz = UzmiBitmap(putanja, "dugme_izlaz_sprajtmapa.gif");
    r_CvetBitmap = UzmiBitmap(putanja, "cvet.gif");
    r_NazivPrograma = UzmiBitmap(putanja, "naziv_aplikacije.gif");
    r_Programer = UzmiBitmap(putanja, "programer.gif");
    r_MeniDa = UzmiBitmap(putanja, "meni_da.gif");
    r_MeniNe = UzmiBitmap(putanja, "meni_ne.gif");
    r_PreIntro = UzmiBitmap(putanja, "preintro.gif");
    //fontovi
    r_Font27 = UzmiBitmap("fntres001.bin","fnt27_b.gif");
    r_Font52 = UzmiBitmap("fntres001.bin","fnt52_b.gif");
    r_Font84 = UzmiBitmap("fntres001.bin","fnt84_b.gif");
}

void cResourceManager::ImgExit()
{
    SDL_FreeSurface(r_IntroPozadina);
    SDL_FreeSurface(r_DugmeStart);
    SDL_FreeSurface(r_IzborPozadina);
    SDL_FreeSurface(r_DugmeIzbor);
    SDL_FreeSurface(r_StapiciPozadina);
    SDL_FreeSurface(r_StapiciBitmap);
    SDL_FreeSurface(r_DugmeIzlaz);
    SDL_FreeSurface(r_CvetBitmap);
    SDL_FreeSurface(r_MeniDa);
    SDL_FreeSurface(r_MeniNe);
    SDL_FreeSurface(r_NazivPrograma);
    SDL_FreeSurface(r_Programer);
    SDL_FreeSurface(r_PreIntro);
    //fontovi
    SDL_FreeSurface(r_Font27);
    SDL_FreeSurface(r_Font52);
    SDL_FreeSurface(r_Font84);
}

SDL_Surface* cResourceManager::UzmiBitmap(char *resourcefilename, char *bitmapfilename)
{
	//Get the bitmap's buffer and size from the resource file
	int filesize = 0;
	char *buffer = UzmiMediju(resourcefilename, bitmapfilename, &filesize);

	//Load the buffer into a surface using RWops
	SDL_RWops *rw = SDL_RWFromMem(buffer, filesize);
	SDL_Surface *temp = IMG_LoadGIF_RW(rw);

	//Release the bitmap buffer memory
	free(buffer);

	//Were we able to load the bitmap?
	if (temp == NULL)
	{
	    FILE *f = fopen("greska.txt","w");
		fprintf(f,"Unable to load bitmap: %s\n", SDL_GetError());
		fclose(f);
		exit(1);
	}

	//Convert the image to optimal display format
	SDL_Surface *image;
	image = SDL_DisplayFormat(temp);

	//Free the temporary surface
	SDL_FreeSurface(temp);

	//Return our loaded image
	return image;
}

Mix_Chunk* cResourceManager::UzmiZvuk(char *resourcefilename, char *soundfilename)
{
        //Get the sound's buffer and size from the resource file
        int filesize = 0;
        char *buffer = UzmiMediju(resourcefilename, soundfilename, &filesize);

        //Load the buffer into a surface using RWops
        SDL_RWops *rw = SDL_RWFromMem(buffer, filesize);
        Mix_Chunk *sound = Mix_LoadWAV_RW(rw, 1);

        //Release the buffer memory
        free(buffer);

        //Return the sound
        return sound;
}

Mix_Music* cResourceManager::UzmiMuziku(char *resourcefilename, char *soundfilename)
{
        //Get the sound's buffer and size from the resource file
        int filesize = 0;
        char *buffer = UzmiMediju(resourcefilename, soundfilename, &filesize);

        //Load the buffer into a surface using RWops
        SDL_RWops *rw = SDL_RWFromMem(buffer, filesize);
        Mix_Music *sound = Mix_LoadMUS_RW(rw);

        //Release the buffer memory
        free(buffer);

        //Return the sound
        return sound;
}

char* cResourceManager::UzmiMediju(char *resourcefilename, char *resourcename, int *filesize)
{
	//Try to open the resource file in question
	int fd = open(resourcefilename, O_RDONLY);
	if (fd < 0)
	{
		perror("Error opening resource file");
		exit(1);
	}

	//Make sure we're at the beginning of the file
	lseek(fd, 0, SEEK_SET);

	//Read the first INT, which will tell us how many files are in this resource
	int numfiles;
	read(fd, &numfiles, sizeof(int));

	//Get the pointers to the stored files
	int *filestart = (int *) malloc(sizeof(int) * numfiles);	// this is probably wrong in the zip
	read(fd, filestart, sizeof(int) * numfiles);

	//Loop through the files, looking for the file in question
	int filenamesize;
	char *buffer;
	int i;
	for(i=0;i<numfiles;i++)
	{
		char *filename;
		//Seek to the location
		lseek(fd, filestart[i], SEEK_SET);
		//Get the filesize value
		read(fd, filesize, sizeof(int));
		//Get the size of the filename string
		read(fd, &filenamesize, sizeof(int));
		//Size the buffer and read the filename
		filename = (char *) malloc(filenamesize + 1);
		read(fd, filename, filenamesize);
		//Remember to terminate the string properly!
		filename[filenamesize] = '\0';
		//Compare to the string we're looking for
		if (strcmp(filename, resourcename) == 0)
		{
			//Get the contents of the file
			buffer = (char *) malloc(*filesize);
			read(fd, buffer, *filesize);
			free(filename);
			break;
		}
		//Free the filename buffer
		free(filename);
	}

	//Release memory
	free(filestart);

	//Close the resource file!
	close(fd);

	//Did we find the file within the resource that we were looking for?
	if (buffer == NULL)
	{
		printf("Unable to find '%s' in the resource file!\n", resourcename);
		exit(1);
	}

	//Return the buffer
	return buffer;
}
