
#include <stdio.h>
#include <unistd.h>
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "../core/gameengine.h"
#include "gamestate.h"
#include "intro.h"
#include "../gui/dugme.h"
#include "glavni_meni.h"


cIntro cIntro::m_Intro;

const int          SCREEN_WIDTH = 640;
const int          SCREEN_HEIGHT = 480;
const int          SCREEN_BPP = 32;

void cIntro::Init(cGameEngine* game)
{
	m_Pozadina = game->UzmiMediju().r_IntroPozadina;

    m_NazivPrograma = game->UzmiMediju().r_NazivPrograma;

    m_Programer = game->UzmiMediju().r_Programer;

	m_FullScr = false;

	Mix_PlayChannel( -1, game->UzmiMediju().r_Odjava, 0 );

	m_ListaObjekata.push_back( new cDugme(207, 318, 215, 102, "Prvo",game) );
}


cIntro::~cIntro()
{
}


void cIntro::Cleanup(cGameEngine* game)
{
	printf("cIntro Cleanup\n");
}

void cIntro::Pause()
{
	printf("cIntro Pause\n");
}

void cIntro::Resume()
{
	printf("cIntro Resume\n");
}

void cIntro::HandleEvents(cGameEngine* game)
{

	while (SDL_PollEvent(&event))
	{

        for(list<cObjekat*>::const_iterator it = m_ListaObjekata.begin(); it != m_ListaObjekata.end(); ++it)
        {
         (*it)->HandleEvents(this->event, game);
        }

		if ( event.type == SDL_QUIT )
		{
				game->Quit();
		}

        if ( event.type == SDL_KEYDOWN )
        {
				if (event.key.keysym.sym == SDLK_ESCAPE)
				{
						game->PushState( cGlavniMeni::Instance() );
				}

				if (event.key.keysym.sym == SDLK_F11)
				{
						//if (m_FullScr)
						if (game->g_CeoEkran)
						{
						game->g_Ekran = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_SWSURFACE);
						
						game->g_CeoEkran = false;
						}
						else
						{
						game->g_Ekran = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_FULLSCREEN);
					
						game->g_CeoEkran = true;
						}
				}
		}

	}
}

void cIntro::Update(cGameEngine* game)
{

}

void cIntro::Draw(cGameEngine* game)
{
    game->CrtajSliku( 0, 0, m_Pozadina, game->g_Ekran );
    game->CrtajSliku( 19, 296, m_Programer, game->g_Ekran );
    game->CrtajSliku( 36, 196, m_NazivPrograma, game->g_Ekran );

    for(list<cObjekat*>::const_iterator it = m_ListaObjekata.begin(); it != m_ListaObjekata.end(); ++it)
        {
         (*it)->Draw(game->g_Ekran);
        }
}



