
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "../core/gameengine.h"
#include "gamestate.h"
#include "serijacija.h"
#include "glavni_meni.h"
#include "../gui/cvet_meta.h"
#include "izbor.h"

cSerijacija cSerijacija::m_Serijacija;

const int          SIRINA_EKRANA = 640;
const int          VISINA_EKRANA = 480;
const int          BPP_EKRANA = 32;

void cSerijacija::Init(cGameEngine* game)
{
    m_Poruka = NULL;
    m_PritisniTaster = NULL;

	m_Pozadina = game->UzmiMediju().r_StapiciPozadina;

    int c_offset = 23;

    for (int i = 0; i < 10; i++)
    {
	   game->g_VektorCvetova.push_back(new cCvetMeta(c_offset, 376, i,game));
       c_offset+=40;
    }

    m_DugmeIzlaz = new cDugmeIzlaz(532, 19, 89, 75,game);

    //stavi pozicije balvana u niz
    int x_offset[] = {30, 80, 130, 180, 230, 280, 330, 380, 430, 480};
    //init random
    srand(time(NULL));
    //izmesaj elemente niza
    for ( int i=0; i<10; i++)
    {
        int r = rand() % 10;
        int temp = x_offset[i];
        x_offset[i] = x_offset[r];
        x_offset[r] = temp;
    }

    //za obelezavanje rednog broja
    int j = 9;
    for (int i = 0; i < 10; i++)
    {
	   game->g_TempBalvanS = new cBalvan;
       game->g_TempBalvanS->PostaviSprajt( &game->g_ClipoviZaBalvane[ i ] );
       game->g_TempBalvanS->Init( x_offset[i],48,game->g_TempBalvanS->UzmiSirinuSprajta(),game->g_TempBalvanS->UzmiVisinuSprajta(),j,game );
	   game->g_VektorBalvana.push_back( game->g_TempBalvanS );
	   j--;
    }

    Mix_FadeInMusic( game->UzmiMediju().r_Muzika, -1, 1000 );

    m_FullScr = false;
    game->g_LevoDugmePritisnuto = false;
    game->g_KrajIgre  = false;

    game->g_BrojKolizija = 0;

    m_Tajmer = new cTajmer;
    m_Tajmer->Pokreni();

    m_Poruka = new cFont84(game);
    m_PritisniTaster = new cFont27(game);
}

void cSerijacija::Cleanup(cGameEngine* game)
{
	printf("cSerijacija Cleanup\n");
    
    delete m_Tajmer;

    if ( game->g_TempBalvan != NULL) game->g_TempBalvan = NULL;
    if ( game->g_TempBalvanS != NULL) game->g_TempBalvanS = NULL;

    game->g_VektorCvetova.clear();
	game->g_VektorBalvana.clear();
	game->g_VektorPoretka.clear();


    Mix_FadeOutMusic( 1000 );
}

void cSerijacija::Pause()
{
	printf("cSerijacija Pause\n");
}

void cSerijacija::Resume()
{
	printf("cSerijacija Resume\n");
}

void cSerijacija::HandleEvents(cGameEngine* game)
{

//igra nije gotova,proveravaj evente
if (!game->g_KrajIgre)
{
	while (SDL_PollEvent(&event))
	{
        m_DugmeIzlaz->HandleEvents(this->event, game);

        for (int i = 0; i < 10; i++) game->g_VektorBalvana[i]->RukovanjeSignalima(this->event,game);

		if ( event.type == SDL_QUIT )
		{
				game->Quit();
		}

        if ( event.type == SDL_KEYDOWN )
        {
				if (event.key.keysym.sym == SDLK_ESCAPE)
				{
						game->PushState( cGlavniMeni::Instance() );
				}
				if (event.key.keysym.sym == SDLK_SPACE)
				{
						game->ChangeState( cSerijacija::Instance() );
				}
				if (event.key.keysym.sym == SDLK_F11)
				{
						if (m_FullScr)
						{
						game->g_Ekran = SDL_SetVideoMode(SIRINA_EKRANA, VISINA_EKRANA, BPP_EKRANA, SDL_SWSURFACE);
						m_FullScr = false;
						}
						else
						{
						game->g_Ekran = SDL_SetVideoMode(SIRINA_EKRANA, VISINA_EKRANA, BPP_EKRANA, SDL_FULLSCREEN);
						m_FullScr = true;
						}
				}
		}
	}
}

if (game->g_KrajIgre)
{
    //igra je gotova,pocni rezervni loop
    while( SDL_PollEvent( &event ) )
    {
        if(event.type == SDL_KEYDOWN)
        {
                //izadji na <Esc>
                if(event.key.keysym.sym == SDLK_ESCAPE)
                {
                    game->PushState( cGlavniMeni::Instance() );
                }

                if(event.key.keysym.sym == SDLK_RETURN)
                {
                    game->ChangeState( cSerijacija::Instance() );
                }
        }

        if( event.type == SDL_MOUSEBUTTONDOWN )
         {
           if( event.button.button == SDL_BUTTON_RIGHT )
            {
                game->ChangeState( cSerijacija::Instance() );            }
         }
        //ako se korisnik odjavi
        if( event.type == SDL_QUIT )
        {
            game->Quit();
        }
    }
  }
}

void cSerijacija::Update(cGameEngine* game)
{
//ako nije kraj igre i levo dugme nije pritisnuto,proveri koliziju
if (!game->g_KrajIgre && !game->g_LevoDugmePritisnuto)
  {
    for (int i = 0; i < 10; i++)
    {
        for (int j =0; j < 10; j++)
        {
        //proveri koliziju ako cvet nije mrtav
        if( game->ProveriKoliziju( game->g_VektorBalvana[i]->UzmiKutiju(),game->g_VektorCvetova[j]->UzmiKutiju()) && !game->g_VektorCvetova[j]->DalJeMrtav())
         {
            Mix_PlayChannel( -1, game->UzmiMediju().r_Ubac, 0 );
            game->g_VektorBalvana[i]->UzmiRedniBrCveta( game->g_VektorCvetova[j]->VratiRedniBroj() );
            //uzmi koord cveta
            //izracunaj koord. balvana u odnosu na cvet
            //i postavi balvan na mesto
            //zatim deaktiviraj cvet i balvan
            game->g_VektorBalvana[i]->PostaviNaPoziciju(game->g_VektorCvetova[j]->UzmiKoordX()-7,
                                                game->g_VektorCvetova[j]->UzmiKoordY()-(game->g_VektorBalvana[i]->UzmiVisinuSprajta()+3));
            game->g_VektorCvetova[j]->Deaktiviraj();
            game->g_VektorBalvana[i]->Deaktiviraj();
            //povecaj broj kolizija za jedan
            game->g_BrojKolizija++;
         }
        }
    }

    if ( game->g_BrojKolizija == 10 && IspravnoPoredjano(game) )
    {
        game->g_KrajIgre  = true;
        Mix_FadeOutMusic( 100 );
        Mix_PlayChannel( -1, game->UzmiMediju().r_Aplauz, 0 );
    }

    if ( game->g_BrojKolizija == 10 && !IspravnoPoredjano(game) )
    {
        game->g_KrajIgre  = true;
        Mix_FadeOutMusic( 100 );
        Mix_PlayChannel( -1, game->UzmiMediju().r_PokusajPonovo, 0 );
    }
  }

}

void cSerijacija::Draw(cGameEngine* game)
{

    game->CrtajSliku( 0, 0, m_Pozadina, game->g_Ekran );

    m_DugmeIzlaz->Draw(game->g_Ekran);

    for ( int i = 0; i < 10; i++) game->g_VektorCvetova[i]->Prikazi( game->g_Ekran, true );

    for ( int i = 0; i < 10; i++) game->g_VektorBalvana[i]->Prikazi( game->g_Ekran );

    if (game->g_KrajIgre && IspravnoPoredjano(game))
     {
        m_Poruka->PrikaziPostepeno("Bravo!", 180, 200,game->g_Ekran,1,game->g_Frejm);
        if ( m_Tajmer->UzmiTikove() & 1024 ) m_PritisniTaster->Prikazi("Pritisni desni taster mi[a",1,100,324,game->g_Ekran);
     }

    if (game->g_KrajIgre && !IspravnoPoredjano(game))
     {
        m_Poruka->PrikaziPostepeno("Gre[ka!", 150, 200,game->g_Ekran,1,game->g_Frejm);
        if ( m_Tajmer->UzmiTikove() & 1024 ) m_PritisniTaster->Prikazi("Pritisni desni taster mi[a",1,100,324,game->g_Ekran);
     }

}

bool cSerijacija::IspravnoPoredjano(cGameEngine* game)
{
    for (int i = 0; i < 10; i++)
    {
            if ( game->g_VektorBalvana[i]->VratiRedniBroj() != game->g_VektorBalvana[i]->VratiRedniBrCveta() ) return false;
    }
    return true;
}



